
#include "LBBCManager.h"

namespace LBBC {

LBBCManager::LBBCManager() 
	: BBC::BBCManager() 
{
	mBillboardCloudSerializer = new KMeansClusterSerializer();
	mLeafDistributionSerializer = new LeafDistributionSerializer();
	mLeafDistributionGenerator = new LeafDistributionGenerator();
	mBillboardCloudGenerator = new KMeansClusterGenerator();
	mBillboardCloudUVMapper = new BBC::BillboardCloudUVMapper();
	mOgreMeshSerializer = new BBC::OgreMeshSerializer();
}

LBBCManager::~LBBCManager() 
{
	if (mOgreMeshSerializer)
	{
		delete mOgreMeshSerializer;
	}
	// Gametools -- BUG: 06/04/2006
	//if (mSampleConfigFile)
	//{
	//	delete mSampleConfigFile;
	//}
	if (mLeafDistributionSerializer)
	{
		delete mLeafDistributionSerializer;
	}
	if (mLeafDistributionGenerator)
	{
		delete mLeafDistributionGenerator;
	}
	if (mBillboardCloudGenerator)
	{
		delete mBillboardCloudGenerator;
	}
	if (mBillboardCloudUVMapper)
	{
		delete mBillboardCloudUVMapper;
	}
	if (mBillboardCloudSerializer)
	{
		delete mBillboardCloudSerializer;
	}
	if (mLeaves)
	{
		delete mLeaves;
	}

}

SampleConfigFile* LBBCManager::getSampleConfigFile()
{
	return mSampleConfigFile;
}

void LBBCManager::setSampleConfigFile(SampleConfigFile *sampleConfigFile)
{
	mSampleConfigFile = sampleConfigFile;
}

void LBBCManager::loadSampleConfigFile(Ogre::String sampleConfigFile)
{
	mSampleConfigFile = new SampleConfigFile();
	mSampleConfigFile->load(sampleConfigFile);
}

void LBBCManager::initialize()
{
	mLeaves = new Leaves();
	BBC::EntityPtr entityLeaves(new BBC::Entity);
	BBC::MeshPtr meshLeaves(new BBC::Mesh(this->loadMesh(mSampleConfigFile->getEntitiesFolder(), mSampleConfigFile->getEntitiesMeshName())) );
	entityLeaves->setMesh(meshLeaves);	
	mLeaves->setEntity(entityLeaves);
	mLeaves->getEntity()->setMesh(meshLeaves);
}

void LBBCManager::createEntityDistribution()
{
	mLeafDistributionGenerator->setDistribution(mLeaves->getDistribution());
	mLeafDistributionGenerator->getEntity()->setMesh(mLeaves->getEntity()->getMesh());	
	mLeafDistributionGenerator->generate();
}

void LBBCManager::saveEntityDistributionSplitted()
{
	BBC::EntityPtr entity;
	entity = BBC::EntityPtr(new BBC::Entity());

	if (mSampleConfigFile->getEntityDistributionSplitted() || mSampleConfigFile->getEntityDistributionMerged())
	{
		BBC::EntityDistribution *entityDistribution;		
		unsigned int numSerializedEntities = 0;
		
		entityDistribution = mLeaves->getDistribution();
		for (unsigned int iEntity = 0; iEntity < entityDistribution->getNumEntities(); iEntity++)
		{				
				BBC::SubEntityPtr subEntity = entityDistribution->getEntity(iEntity)->getSubEntity(0);

				if (iEntity == 0)
				{
					// delete the default constructed subentity...
					entity->removeSubEntity(0);

					// Add our first subentity...
					entity->addSubEntity(subEntity);					
				}
				else
				{
					entity->addSubEntity(subEntity);
				}

				entity->getSubEntity(numSerializedEntities)->setMaterialName(mSampleConfigFile->getEntityDistributionMergedMaterialName());
				numSerializedEntities++;			
		}		

		entity->setSubEntitiesDistinctVertexColours();
		
		if (mSampleConfigFile->getEntityDistributionSplitted())
		{
			mOgreMeshSerializer->setEntity(entity);
			mOgreMeshSerializer->exportMesh(mSampleConfigFile->getEntityClustersFolder() +
				mSampleConfigFile->getEntityClustersSplittedMeshName(), false, false);
			
			OBA::OgreBase::getSingleton().getMeshSerializer()->exportMesh(entity->getMesh()->get(), mSampleConfigFile->getEntityDistributionFolder() + mSampleConfigFile->getEntityDistributionSplittedMeshName());
			//OBA::OgreBase::getSingleton().getXMLMeshSerializer()->exportMesh(entity->getMesh().get(), mSampleConfigFile->getEntityClustersFolder() + mSampleConfigFile->getEntitiesMeshPrefixName() + "Splitted.mesh" + ".xml");			
		}
	}

	if (mSampleConfigFile->getEntityDistributionMerged())
	{
		entity->mergeSubEntities();
		entity->getSubEntity(0)->setMaterialName(mSampleConfigFile->getEntityDistributionMergedMaterialName());
	
		mOgreMeshSerializer->setEntity(entity);
		mOgreMeshSerializer->exportMesh(mSampleConfigFile->getEntityClustersFolder() +
			mSampleConfigFile->getEntityClustersMergedMeshName(), false, false);
		
		OBA::OgreBase::getSingleton().getMeshSerializer()->exportMesh((Ogre::Mesh*)entity->getMesh().get(), mSampleConfigFile->getEntityDistributionFolder() + mSampleConfigFile->getEntityClustersMergedMeshName());
		//OBA::OgreBase::getSingleton().getXMLMeshSerializer()->exportMesh(entity->getMesh(), mSampleConfigFile->getEntityClustersFolder() + mSampleConfigFile->getEntitiesMeshPrefixName() + "Merged.mesh" + ".xml");				
	}
}

void LBBCManager::saveEntityDistributionXML()
{
	TiXmlDocument *document = mLeafDistributionSerializer->create(
									mSampleConfigFile->getEntityDistributionFolder() +
									mSampleConfigFile->getEntityDistributionXMLName());
	mLeafDistributionSerializer->setLeafDistribution(mLeaves->getDistribution());
	mLeafDistributionSerializer->writeLeafDistribution(document);	
	mLeafDistributionSerializer->save();
}

void LBBCManager::loadEntityDistribution()
{
	TiXmlDocument *document = mLeafDistributionSerializer->load(
									mSampleConfigFile->getEntityDistributionFolder() +
									mSampleConfigFile->getEntityDistributionXMLName());
	mLeafDistributionSerializer->setLeafDistribution(mLeaves->getDistribution());
	mLeafDistributionSerializer->getEntity()->setMesh(mLeaves->getEntity()->getMesh());
	mLeafDistributionSerializer->readLeafDistribution(document);

	BBC::EntityPtr entity;
	BBC::EntityDistribution *entityDistribution;

	BBC::MeshPtr pMeshLeavesSplitted(new BBC::Mesh(this->loadMesh(mSampleConfigFile->getEntitiesFolder(), mSampleConfigFile->getEntityDistributionSplittedMeshName())));
	entity = BBC::EntityPtr(new BBC::Entity());
	entity->setMesh(pMeshLeavesSplitted);

	entity->loadMesh(false);
	entityDistribution = mLeaves->getDistribution();

	for (unsigned int iSubEntity = 0; iSubEntity < entity->getNumSubEntities(); iSubEntity++)
	{	
		BBC::EntityPtr entitySample;
		BBC::SubEntityPtr subEntity;
		BBC::SubEntityPtr subEntitySample;
			
		subEntity = entity->getSubEntity(iSubEntity);
		entitySample = entityDistribution->getEntity(iSubEntity);
		subEntitySample = entitySample->getSubEntity(0);
		entitySample->removeSubEntity(0);				

		// Add our first subentity...
		entitySample->addSubEntity(subEntity);	
	}
}

void LBBCManager::generateEntityDistribution()
{
	if (mSampleConfigFile->getEntityDistributionGeneration())
	{
		createEntityDistribution();
		saveEntityDistributionSplitted();
		saveEntityDistributionXML();
	}
}

void LBBCManager::saveBillboardCloudGrouped()
{
	for (unsigned int iSubEntity = 0; iSubEntity < mLeaves->getBillboardCloud()->getEntity()->getNumSubEntities(); iSubEntity++)
	{
		BBC::SubEntityPtr subEntity = mLeaves->getBillboardCloud()->getEntity()->getSubEntity(iSubEntity);
		subEntity->setMaterialName(mSampleConfigFile->getBillboardCloudGroupedMaterialName());
	}
	mOgreMeshSerializer->setEntity(mLeaves->getBillboardCloud()->getEntity());
   
	Ogre::LogManager::getSingleton().logMessage("Num.SubEntities:" + Ogre::StringConverter::toString(mLeaves->getBillboardCloud()->getEntity()->getNumSubEntities()));
	mOgreMeshSerializer->exportMesh(mSampleConfigFile->getBillboardCloudFolder() +
				mSampleConfigFile->getBillboardCloudGroupedMeshName(),false, false);
	OBA::OgreBase::getSingleton().getMeshSerializer()->exportMesh(mLeaves->getBillboardCloud()->getEntity()->getMesh()->get(), mSampleConfigFile->getBillboardCloudFolder() + mSampleConfigFile->getBillboardCloudGroupedMeshName());
	OBA::OgreBase::getSingleton().getXMLMeshSerializer()->exportMesh(mLeaves->getBillboardCloud()->getEntity()->getMesh()->get(), mSampleConfigFile->getBillboardCloudFolder() + mSampleConfigFile->getBillboardCloudGroupedMeshName() + ".xml");
}


void LBBCManager::saveBillboardCloudSplitted()
{	
	for (unsigned int iSubEntity = 0; iSubEntity < mLeaves->getBillboardCloud()->getEntity()->getNumSubEntities(); iSubEntity++)
	{
		BBC::SubEntityPtr subEntity = mLeaves->getBillboardCloud()->getEntity()->getSubEntity(iSubEntity);
		subEntity->setMaterialName(mSampleConfigFile->getBillboardCloudSplittedMaterialName());
	}

	mOgreMeshSerializer->setEntity(mLeaves->getBillboardCloud()->getEntity());

	Ogre::LogManager::getSingleton().logMessage("Num.SubEntities:" + Ogre::StringConverter::toString(mLeaves->getBillboardCloud()->getEntity()->getNumSubEntities()));

	mOgreMeshSerializer->exportMesh(mSampleConfigFile->getBillboardCloudFolder() +
				mSampleConfigFile->getBillboardCloudSplittedMeshName(),false, false);

	OBA::OgreBase::getSingleton().getMeshSerializer()->exportMesh(mLeaves->getBillboardCloud()->getEntity()->getMesh()->get(), mSampleConfigFile->getBillboardCloudFolder() + mSampleConfigFile->getBillboardCloudSplittedMeshName());
	OBA::OgreBase::getSingleton().getXMLMeshSerializer()->exportMesh(mLeaves->getBillboardCloud()->getEntity()->getMesh()->get(), mSampleConfigFile->getBillboardCloudFolder() + mSampleConfigFile->getBillboardCloudSplittedMeshName() + ".xml");
}

void LBBCManager::saveBillboardCloudMerged()
{	
	for (unsigned int iSubEntity = 0; iSubEntity < mLeaves->getBillboardCloud()->getEntity()->getNumSubEntities(); iSubEntity++)
	{
		BBC::SubEntityPtr subEntity = mLeaves->getBillboardCloud()->getEntity()->getSubEntity(iSubEntity);
		subEntity->setMaterialName(mSampleConfigFile->getBillboardCloudGroupedMaterialName());
	}
	mOgreMeshSerializer->setEntity(mLeaves->getBillboardCloud()->getEntity());
   
	Ogre::LogManager::getSingleton().logMessage("Num.SubEntities:" + Ogre::StringConverter::toString(mLeaves->getBillboardCloud()->getEntity()->getNumSubEntities()));
	mOgreMeshSerializer->exportMesh(mSampleConfigFile->getBillboardCloudFolder() +
				mSampleConfigFile->getBillboardCloudMergedMeshName(),false, false);
	OBA::OgreBase::getSingleton().getMeshSerializer()->exportMesh(mLeaves->getBillboardCloud()->getEntity()->getMesh()->get(), mSampleConfigFile->getBillboardCloudFolder() + mSampleConfigFile->getBillboardCloudMergedMeshName());	
	OBA::OgreBase::getSingleton().getXMLMeshSerializer()->exportMesh(mLeaves->getBillboardCloud()->getEntity()->getMesh()->get(), mSampleConfigFile->getBillboardCloudFolder() + mSampleConfigFile->getBillboardCloudMergedMeshName() + ".xml");	
}

void LBBCManager::saveBillboardCloudXML()
{
	TiXmlDocument *document2 = mBillboardCloudSerializer->create(
									mSampleConfigFile->getBillboardCloudFolder() + 
									mSampleConfigFile->getBillboardCloudSplittedXMLName());
	mBillboardCloudSerializer->setBillboardCloud(mLeaves->getBillboardCloud());
	mBillboardCloudSerializer->writeBillboardCloud(document2, false);	
	mBillboardCloudSerializer->save();
}

void LBBCManager::saveBillboardCloudGroupedXML()
{
	TiXmlDocument *document2 = mBillboardCloudSerializer->create(
									mSampleConfigFile->getBillboardCloudFolder() + 
									mSampleConfigFile->getBillboardCloudGroupedXMLName());
	mBillboardCloudSerializer->setBillboardCloud(mLeaves->getBillboardCloud());
	mBillboardCloudSerializer->writeBillboardCloud(document2, true);	
	mBillboardCloudSerializer->save();
}

void LBBCManager::saveBillboardCloudMergedXML()
{
	TiXmlDocument *document2 = mBillboardCloudSerializer->create(
									mSampleConfigFile->getBillboardCloudFolder() + 
									mSampleConfigFile->getBillboardCloudMergedXMLName());
	mBillboardCloudSerializer->setBillboardCloud(mLeaves->getBillboardCloud());
	mBillboardCloudSerializer->writeBillboardCloud(document2, true);	
	mBillboardCloudSerializer->save();
}

void LBBCManager::loadBillboardCloudXML()
{
	TiXmlDocument *document = mBillboardCloudSerializer->load(
									mSampleConfigFile->getBillboardCloudFolder() +
									mSampleConfigFile->getBillboardCloudSplittedXMLName());

	mBillboardCloudSerializer->setBillboardCloud(mLeaves->getBillboardCloud());
	mBillboardCloudSerializer->readBillboardCloud(document, false);
}

void LBBCManager::loadBillboardCloudGroupedXML()
{
	TiXmlDocument *document = mBillboardCloudSerializer->load(
									mSampleConfigFile->getBillboardCloudFolder() +
									mSampleConfigFile->getBillboardCloudGroupedXMLName());
	
	mBillboardCloudSerializer->setBillboardCloud(mLeaves->getBillboardCloud());
	mBillboardCloudSerializer->readBillboardCloud(document, true);
}

void LBBCManager::loadBillboardCloudMergedXML()
{
	TiXmlDocument *document = mBillboardCloudSerializer->load(
									mSampleConfigFile->getBillboardCloudFolder() +
									mSampleConfigFile->getBillboardCloudMergedXMLName());

	mBillboardCloudSerializer->setBillboardCloud(mLeaves->getBillboardCloud());
	mBillboardCloudSerializer->readBillboardCloud(document, true);
}

void LBBCManager::saveEntityClusters()
{
	BBC::BillboardClusterDataPtr billboardClusterData;
	BBC::EntityClusterPtr entityCluster;
	unsigned int numSerializedBillboards = 0;	

	if (mSampleConfigFile->getEntityClustersSplittedClusters())
	{

		for (unsigned int iBillboard = 0; iBillboard < mLeaves->getBillboardCloud()->getNumBillboards(); iBillboard++)
		{
			billboardClusterData = mLeaves->getBillboardCloud()->getBillboard(iBillboard)->getBillboardClusterData();
			entityCluster = billboardClusterData->getEntityCluster();
			Ogre::LogManager::getSingleton().logMessage("iBillboard: " + Ogre::StringConverter::toString(iBillboard));
			if (entityCluster->getNumEntitiesClusterData() > 0)
			{					
				BBC::EntityPtr entityClusterized = entityCluster->getEntity();
				entityClusterized->getSubEntity(0)->setMaterialName(mSampleConfigFile->getEntityClustersSplittedMaterialName());
				mOgreMeshSerializer->setEntity(entityClusterized);
				mOgreMeshSerializer->exportMesh(mSampleConfigFile->getEntityDistributionFolder() +
					mSampleConfigFile->getEntityClustersSplittedMeshName() +
					Ogre::StringConverter::toString(numSerializedBillboards) + ".mesh", !(mSampleConfigFile->getEntityClustersSplittedClusters()), false);
				OBA::OgreBase::getSingleton().getMeshSerializer()->exportMesh(entityClusterized->getMesh()->get(),
					mSampleConfigFile->getEntityClustersFolder() +
					mSampleConfigFile->getEntityClustersSplittedMeshName() +
					Ogre::StringConverter::toString(numSerializedBillboards) + ".mesh");
				//OBA::OgreBase::getSingleton().getXMLMeshSerializer()->exportMesh(entityClusterized->getMesh(), entityClusterized->getMesh()->getName() + ".xml");

				numSerializedBillboards++;
			}
		}
	}
	numSerializedBillboards = 0;

	if (mSampleConfigFile->getEntityClustersMergedClusters())
	{
		BBC::EntityPtr entity;
		entity = BBC::EntityPtr(new BBC::Entity());

		for (unsigned int iBillboard = 0; iBillboard < mLeaves->getBillboardCloud()->getNumBillboards(); iBillboard++)
		{
			billboardClusterData = mLeaves->getBillboardCloud()->getBillboard(iBillboard)->getBillboardClusterData();
			entityCluster = billboardClusterData->getEntityCluster();

			if (entityCluster->getNumEntitiesClusterData() > 0)
			{			
				BBC::EntityPtr entityClusterized = entityCluster->getEntity();
				entityClusterized->getSubEntity(0)->setMaterialName(mSampleConfigFile->getEntityClustersMergedMaterialName());
				BBC::SubEntityPtr subEntity = entityClusterized->getSubEntity(0);

				if (numSerializedBillboards == 0)
				{
					// delete the default constructed subentity...
					entity->removeSubEntity(0);

					// Add our first subentity...
					entity->addSubEntity(subEntity);
				}
				else
				{
					entity->addSubEntity(subEntity);
				}

				numSerializedBillboards++;
			}
		}
		
		Ogre::LogManager::getSingleton().logMessage(" Mesh Generation --" + Ogre::StringConverter::toString(numSerializedBillboards));
		
		mOgreMeshSerializer->setEntity(entity);
		mOgreMeshSerializer->exportMesh(mSampleConfigFile->getEntityClustersFolder() +
			mSampleConfigFile->getEntityClustersMergedMeshName(), false, false);
		
		OBA::OgreBase::getSingleton().getMeshSerializer()->exportMesh(entity->getMesh()->get(), mSampleConfigFile->getEntityClustersFolder() + mSampleConfigFile->getEntityClustersMergedMeshName());
		//OBA::OgreBase::getSingleton().getXMLMeshSerializer()->exportMesh(entity->getMesh(), mSampleConfigFile->getEntityClustersFolder() + mSampleConfigFile->getEntityClustersMeshPrefixName() + "Merged.mesh" + ".xml");
	}
}

void LBBCManager::createEntityClusters()
{
	mBillboardCloudGenerator->setDistribution(mLeaves->getDistribution());
	mBillboardCloudGenerator->setBillboardCloud(mLeaves->getBillboardCloud());
	mBillboardCloudGenerator->setMaxNumBillboards(mSampleConfigFile->getEntityClustersMaxClusters());
	((KMeansClusterGenerator*)mBillboardCloudGenerator)->setAlpha(mSampleConfigFile->getEntityClustersAlpha());
	((KMeansClusterGenerator*)mBillboardCloudGenerator)->setNumIterations(mSampleConfigFile->getEntityClustersNumIterations());
	((KMeansClusterGenerator*)mBillboardCloudGenerator)->generate();
}

void LBBCManager::saveSingleEntity()
{
	BBC::EntityPtr entityLeaf = mLeaves->getDistribution()->getEntity(2);
	entityLeaf->getSubEntity(0)->setMaterialName(mSampleConfigFile->getEntitySampleMaterialName());
	mOgreMeshSerializer->setEntity(entityLeaf);
	mOgreMeshSerializer->exportMesh(mSampleConfigFile->getEntityDistributionFolder() +
		mSampleConfigFile->getEntitySampleMeshName(), false, false);
	OBA::OgreBase::getSingleton().getMeshSerializer()->exportMesh(entityLeaf->getMesh()->get(), mSampleConfigFile->getEntityDistributionFolder() + mSampleConfigFile->getEntitySampleMeshName());
	OBA::OgreBase::getSingleton().getXMLMeshSerializer()->exportMesh(entityLeaf->getMesh()->get(), mSampleConfigFile->getEntityDistributionFolder() + mSampleConfigFile->getEntitySampleMeshName() + ".xml");	
}

void LBBCManager::generateEntityClusters()
{
	createEntityClusters();
	saveEntityClusters();	
}

void LBBCManager::generateBillboardCloud()
{

	if (mSampleConfigFile->getBillboardCloudGroupedBillboards())
	{				
		mLeaves->getBillboardCloud()->generateBillboardCloud(false);
		mLeaves->getBillboardCloud()->initializeBillboardCloudGroups(mSampleConfigFile->getBillboardCloudGroups());
		
		if (mSampleConfigFile->getBillboardCloudDiffuseColorBillboardGroupTextureAtlasGeneration())
		{
			mBillboardCloudUVMapper->shutdown();
			mBillboardCloudUVMapper->setBillboardCloud(mLeaves->getBillboardCloud());
			mBillboardCloudUVMapper->setTextureWidth(mSampleConfigFile->getBillboardCloudDiffuseColorBillboardGroupTextureSize());
			mBillboardCloudUVMapper->setTextureHeight(mSampleConfigFile->getBillboardCloudDiffuseColorBillboardGroupTextureSize());
			mBillboardCloudUVMapper->setTextureAtlasWidth(mSampleConfigFile->getBillboardCloudDiffuseColorBillboardGroupTextureAtlasSize());
			mBillboardCloudUVMapper->setTextureAtlasHeight(mSampleConfigFile->getBillboardCloudDiffuseColorBillboardGroupTextureAtlasSize());
			mBillboardCloudUVMapper->initialize();			
			mBillboardCloudUVMapper->generateTextureAtlasTextureCoords(false, mSampleConfigFile->getBillboardCloudGroupedBillboards(), mSampleConfigFile->getBillboardCloudGroups()); 
			
			mLeaves->getBillboardCloud()->generateBillboardCloudGroups();
			//saveBillboardCloudGrouped();
			//saveBillboardCloudGroupedXML();
		}
	}

	if (mSampleConfigFile->getBillboardCloudMergedBillboards())
	{
		mLeaves->getBillboardCloud()->generateBillboardCloud(false);

		mLeaves->getBillboardCloud()->initializeBillboardCloudGroups(1);

		if (mSampleConfigFile->getBillboardCloudDiffuseColorMergedBillboardsTextureAtlasGeneration())
		{
			mBillboardCloudUVMapper->shutdown();
			mBillboardCloudUVMapper->setBillboardCloud(mLeaves->getBillboardCloud());
			mBillboardCloudUVMapper->setTextureWidth(mSampleConfigFile->getBillboardCloudDiffuseColorMergedBillboardsTextureSize());
			mBillboardCloudUVMapper->setTextureHeight(mSampleConfigFile->getBillboardCloudDiffuseColorMergedBillboardsTextureSize());
			mBillboardCloudUVMapper->setTextureAtlasWidth(mSampleConfigFile->getBillboardCloudDiffuseColorMergedBillboardsTextureAtlasSize());
			mBillboardCloudUVMapper->setTextureAtlasHeight(mSampleConfigFile->getBillboardCloudDiffuseColorMergedBillboardsTextureAtlasSize());
			mBillboardCloudUVMapper->initialize();
			mBillboardCloudUVMapper->generateTextureAtlasTextureCoords(false, mSampleConfigFile->getBillboardCloudGroupedBillboards(), 1); 

			mLeaves->getBillboardCloud()->generateBillboardCloudGroups();
			saveBillboardCloudMerged();
			saveBillboardCloudMergedXML();
		}
	}

	if (mSampleConfigFile->getBillboardCloudSplittedBillboards())
	{		
		mLeaves->getBillboardCloud()->generateBillboardCloud(false);
		//if (mSampleConfigFile->getBillboardCloudDiffuseColorTextureAtlasGeneration())
		//{
		//	mBillboardCloudUVMapper->shutdown();
		//	mBillboardCloudUVMapper->setBillboardCloud(mLeaves->getBillboardCloud());
		//	mBillboardCloudUVMapper->setTextureWidth(mSampleConfigFile->getBillboardCloudDiffuseColorTextureSize());
		//	mBillboardCloudUVMapper->setTextureHeight(mSampleConfigFile->getBillboardCloudDiffuseColorTextureSize());
		//	mBillboardCloudUVMapper->setTextureAtlasWidth(mSampleConfigFile->getBillboardCloudDiffuseColorTextureAtlasSize());
		//	mBillboardCloudUVMapper->setTextureAtlasHeight(mSampleConfigFile->getBillboardCloudDiffuseColorTextureAtlasSize());
		//	mBillboardCloudUVMapper->initialize();
		//	mBillboardCloudUVMapper->generateTextureAtlasTextureCoords(!mSampleConfigFile->getBillboardCloudSplittedBillboards(),false); 
		//}
		saveBillboardCloudSplitted();
		saveBillboardCloudXML();
	}
}

Leaves* LBBCManager::getLeaves()
{
	return mLeaves;
}

}
